@@ -12,12 +12,9 @@ from .utils import (  # the util stuff
     base64ify,
     debase64ify,
     ensure_bytes,
-    ensure_unicode,
     ensure_valid_filename,
-    serve_once,
     sha256ify,
     snake_caseify,
-    start_daemon,
 )
 
 name = "enochecker"
