@@ -44,7 +44,7 @@ if TYPE_CHECKING:  # pragma: no cover
     import requests
 
 DEFAULT_TIMEOUT: float = 30
-TIME_BUFFER: float = 5  # time in seconds we try to finish earlier
+TIME_BUFFER: float = 2  # time in seconds we try to finish earlier
 
 # Global cache for all stored dicts.  TODO: Prune this at some point?
 global_db_cache: Dict[str, Union[StoredDict, NoSqlDict]] = {}
@@ -178,7 +178,7 @@ class BaseChecker(metaclass=_CheckerMeta):
         self.storage_dir = storage_dir
 
         self._setup_logger()
-        if use_db_cache:
+        if use_db_cache and not os.getenv("MONGO_ENABLED"):
             self._active_dbs: Dict[str, Union[NoSqlDict, StoredDict]] = global_db_cache
         else:
             self._active_dbs = {}
@@ -494,6 +494,8 @@ class BaseChecker(metaclass=_CheckerMeta):
     def exploit(self) -> str:
         """
         Use this method strictly for testing purposes.
+        It should run exploits just like a real team would.
+        It may not use stored data.
 
         Will hopefully not be called during the actual CTF.
 
@@ -545,6 +547,10 @@ class BaseChecker(metaclass=_CheckerMeta):
                 Manual locking ist still possible.
         :return: A dict that will be self storing. Alternatively,
         """
+        if self.method == CheckerMethod.EXPLOIT:
+            raise EnoException(
+                "The exploit method must not rely on stored data! Database access prevented."
+            )
         try:
             db = self._active_dbs[name]
             db.logger = self.logger
@@ -677,7 +683,7 @@ class BaseChecker(metaclass=_CheckerMeta):
         else:
 
             def timeout_fun() -> float:
-                return self.time_remaining / 2
+                return self.time_remaining
 
         if port is None:
             port = self.port
@@ -831,7 +837,7 @@ class BaseChecker(metaclass=_CheckerMeta):
         kwargs.setdefault("allow_redirects", False)
         url = self._sanitize_url(route, port, scheme)
         if timeout is None:
-            timeout = self.time_remaining / 2
+            timeout = self.time_remaining
         self.debug(
             "Request: {} {} with params: {} and {} secs timeout.".format(
                 method, url, params, timeout
@@ -848,6 +854,7 @@ class BaseChecker(metaclass=_CheckerMeta):
 def run(
     checker_cls: Type[BaseChecker],
     args: Optional[Sequence[str]] = None,
+    **kwargs: Any,
 ) -> Optional[CheckerResult]:
     """
     Run a checker, either from cmdline or as uwsgi script.
@@ -862,12 +869,15 @@ def run(
         flask_app = init_service(
             checker_cls, disable_json_logging=parsed.disable_json_logging
         )
-        flask_app.run(host="::", debug=True, port=parsed.listen_port)
+        kwargs.setdefault("host", "::")
+        kwargs.setdefault("debug", True)
+        kwargs.setdefault("port", parsed.listen_port)
+        flask_app.run(**kwargs)
         return None
     else:
         task_message = task_message_from_namespace(parsed)
         result = checker_cls(
             task_message, json_logging=(not parsed.disable_json_logging)
-        ).run()
+        ).run(**kwargs)
         print(f"Checker run resulted in Result: {result.result.name}")
         return result
