@@ -5,6 +5,7 @@ import hashlib
 import logging
 import re
 import socket
+from functools import wraps
 from typing import (
     Any,
     Callable,
@@ -206,6 +207,13 @@ class SimpleSocket(pwnlib.tubes.remote.remote):
             self.logger = utilslogger
         self.timeout_fun: Optional[Callable[[], float]] = timeout_fun
 
+    @wraps(pwnlib.tubes.remote.remote.recv)
+    def recv(self, *args: Tuple[Any, ...], **kwargs: Dict[str, Any]) -> bytes:
+        data = super().recv(*args, **kwargs)
+        if data == b"":
+            raise EOFError("Recv Timeout")
+        return data
+
     @property
     def current_default_timeout(self) -> float:
         """
